<?php
/** cachextra.admin.inc --- 
 *
 * Copyright (C) 2009 PhoeniX11 Kayo
 *
 * Author: PhoeniX11 Kayo <phoenix11@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

function cachextra_config_common () {
  return array(
    'cache-by-host' => array(
      '#type' => 'checkbox',
      '#title' => t('Caching contents by hosts'),
      '#description' => t('Host will be discounted in cache tags.'),
      /*'#default_value' => module_exists('domain'),*/
    ),
    'cache-by-rids' => array(
      '#type' => 'checkbox',
      '#title' => t('Caching contents by roles'),
      '#description' => t('Roles will be discounted in cache tags.'),
    ),
  );
}

function cachextra_config_debug () {
  return array(
    'debug' => array(
      '#type' => 'checkbox',
      '#title' => t('Debug mode'),
      '#description' => t('Enables displaying debug info for users with %priv privilege. Also, in this case, debug info will be displayed if variable CACHEXTRA_DEBUG has been given by GET or POST request.', array('%priv' => t('debug cachextra'))),
    ),
    'debug-js' => array(
      '#type' => 'checkbox',
      '#title' => t('Enable ES in debug mode'),
      '#description' => t('Enables ECMAScript for better displaying of debug info.'),
    ),
  );
}

function cachextra_cachextra_conf () {
  if (module_exists('node'))
    $groups['node'] = array(
      '#title' => t('Caching settings for node display'),
      'node-page-view' => array(
        '#type' => 'checkbox',
        '#title' => t('Caching pages of nodes (/node/*)'),
      ),
      'node-page-default' => array(
        '#type' => 'checkbox',
        '#title' => t('Caching default page of nodes (/node)'),
      ),
      'node-page-teaser' => array(
        '#type' => 'checkbox',
        '#title' => t('Caching each node for default page of nodes'),
        '#description' => t('Makes updating default page of nodes many fastest.'),
      ),
      'node-feed' => array(
        '#type' => 'checkbox',
        '#title' => t('Caching feed page (rss.xml)'),
      ),
      /*
      'node-feed-node' => array(
        '#type' => 'checkbox',
        '#title' => t('Caching each node for feed page'),
        '#description' => t('Makes updating feed page of nodes many fastest.'),
      ),
      */
      'comments-page-view' => array(
        '#type' => 'checkbox',
        '#title' => t('Caching comments on node page'),
      ),
    );
  
  if (module_exists('taxonomy'))
    $groups['term'] = array(
      '#title' => t('Caching settings for taxonomy display'),
      'taxonomy-term-page' => array(
        '#type' => 'checkbox',
        '#title' => t('Caching pages of nodes for taxonomy terms (/taxonomy/term/*)'),
      ),
      /*
      'taxonomy-term-page-node' => array(
        '#type' => 'checkbox',
        '#title' => t('Caching each node for pages of nodes for taxonomy terms'),
        '#description' => t('Makes updating pages of nodes by taxonomy term many fastest.'),
      ),
      */
      'taxonomy-term-feed' => array(
        '#type' => 'checkbox',
        '#title' => t('Caching feeds of nodes for taxonomy terms (/taxonomy/term/*/feed)'),
      ),
      /*
      'taxonomy-term-feed-node' => array(
        '#type' => 'checkbox',
        '#title' => t('Caching each node for feeds of nodes for taxonomy terms'),
        '#description' => t('Makes updating feeds of nodes by taxonomy term many fastest.'),
      ),
      */
    );
  
  return $groups;
}

function cachextra_invoke_conf () {
  $groups = array();
  $hook = 'cachextra_conf';
  foreach (module_implements($hook) as $module) {
    $function = $module.'_'.$hook;
    $groups += $function();
  }
  return $groups;
}

function cachextra_settings () {
  /* common */
  $form['common'] = array(
    '#type' => 'fieldset',
    '#title' => t('Common caching settings'),
  );
  foreach (cachextra_config_common() as $name => $field) {
    $field += array(
      '#parents' => array('cachextra', $name),
      '#default_value' => cachextra_config($name),
    );
    $form['common'][$name] = $field;
  }
  /* conf */
  foreach (cachextra_invoke_conf() as $num => $group) {
    $group += array(
      '#type' => 'fieldset',
    );
    foreach (element_children($group) as $option) {
      $group[$option] += array(
        '#parents' => array('cachextra_opt', $option),
        '#default_value' => cachextra_option($option),
      );
    }
    $form['group_'.$num] = $group;
  }
  /* debug */
  $form['debug'] = array(
    '#type' => 'fieldset',
    '#title' => t('Cache debugging settings'),
  );
  foreach (cachextra_config_debug() as $name => $field) {
    $field += array(
      '#parents' => array('cachextra', $name),
      '#default_value' => cachextra_config($name),
    );
    $form['debug'][$name] = $field;
  }
  
  $form['clear_cache'] = array(
    '#type' => 'fieldset',
    '#title' => t('Clear cachextra cached data'),
    '#description' => t('Caching data improves performance, but may cause problems while troubleshooting new modules, themes, or translations, if outdated information has been cached. To refresh all cached data on your site, click the button below. <em>Warning: high-traffic sites will experience performance slowdowns while cached data is rebuilt.</em>'),
  );
  
  $form['clear_cache']['clear_cachextra'] = array(
    '#type' => 'submit',
    '#value' => t('Clear cachextra cached data'),
    '#submit' => array('cachextra_clear_cache_submit'),
  );
  
  return system_settings_form($form);
}

function cachextra_clear_cache_submit(&$form_state, $form) {
  //cache_clear_all(NULL, CACHEXTRA_BIN);
  cache_clear_all('*', CACHEXTRA_BIN, TRUE);
  drupal_set_message(t('Cachextra caches cleared.'));
}
