<?php
/** cachextra.pages.inc --- 
 *
 * Copyright (C) 2009 PhoeniX11 Kayo
 *
 * Author: PhoeniX11 Kayo <phoenix11@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

/**
 * A generic function for generating RSS feeds from a set of nodes.
 *
 * @param $nids
 *   An array of node IDs (nid). Defaults to FALSE so empty feeds can be
 *   generated with passing an empty array, if no items are to be added
 *   to the feed.
 * @param $channel
 *   An associative array containing title, link, description and other keys.
 *   The link should be an absolute URL.
 */

function cachextra_node_feed ($nids = FALSE, $channel = array()) {
  return cachextra('node-feed', array('nids' => $nids, 'channel' => $channel), 'cachextra_node_feed_real', array($nids, $channel));
}

function cachextra_node_page_default($cond = 'n.promote = 1') {
  return cachextra('node-page-default', array('page' => $_GET['page'], 'cond' => $cond), 'cachextra_node_page_default_real', array($cond));
}

/**
 * Menu callback; Generate a listing of promoted nodes.
 */

function cachextra_node_page_default_real($cond = 'n.promote = 1') {
  $result = pager_query(db_rewrite_sql('SELECT n.nid, n.sticky, n.created FROM {node} n WHERE '.$cond.' AND n.status = 1 ORDER BY n.sticky DESC, n.created DESC'), variable_get('default_nodes_main', 10));
  
  $output = '';
  for (; $node = db_fetch_object($result); )
    $output .= cachextra('node-page-teaser', array('nid' => $node->nid),
                         'cachextra_node_page_teaser_real', array($node->nid));
  
  if ($output) {
    $feed_url = url('rss.xml', array('absolute' => TRUE));
    drupal_add_feed($feed_url, variable_get('site_name', 'Drupal') .' '. t('RSS'));
    $output .= theme('pager', NULL, variable_get('default_nodes_main', 10));
  }
  else {
    return node_page_default();
  }
  drupal_set_title('');
  
  return $output;
}

function cachextra_node_page_teaser_real($nid) {
  return node_view(cachextra_node_load($nid), 1);
}

function cachextra_node_feed_real($nids = FALSE, $channel = array()) {
  global $base_url, $language;
  
  if ($nids === FALSE) {
    $nids = array();
    $result = db_query_range(db_rewrite_sql('SELECT n.nid, n.created FROM {node} n WHERE n.promote = 1 AND n.status = 1 ORDER BY n.created DESC'), 0, variable_get('feed_default_items', 10));
    while ($row = db_fetch_object($result)) {
      $nids[] = $row->nid;
    }
  }
  
  $item_length = variable_get('feed_item_length', 'teaser');
  $namespaces = array('xmlns:dc' => 'http://purl.org/dc/elements/1.1/');
  
  $items = '';
  foreach ($nids as $nid) {
    $items .= cachextra('node-feed-teaser', array('nid' => $nid), 'cachextra_node_feed_teaser_real', array($nid));
  }
  
  $channel_defaults = array(
    'version'     => '2.0',
    'title'       => variable_get('site_name', 'Drupal'),
    'link'        => $base_url,
    'description' => variable_get('site_mission', ''),
    'language'    => $language->language
  );
  $channel = array_merge($channel_defaults, $channel);

  $output = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
  $output .= "<rss version=\"". $channel["version"] ."\" xml:base=\"". $base_url ."\" ". drupal_attributes($namespaces) .">\n";
  $output .= format_rss_channel($channel['title'], $channel['link'], $channel['description'], $items, $channel['language']);
  $output .= "</rss>\n";

  drupal_set_header('Content-Type: application/rss+xml; charset=utf-8');
  print $output;
}

function cachextra_node_feed_teaser_real($nid){
  // Load the specified node:
  $item = cachextra_node_load($nid);
  $item->build_mode = NODE_BUILD_RSS;
  $item->link = url("node/$nid", array('absolute' => TRUE));
  
  if ($item_length != 'title') {
    $teaser = ($item_length == 'teaser') ? TRUE : FALSE;
    
    // Filter and prepare node teaser
    if (node_hook($item, 'view')) {
      $item = node_invoke($item, 'view', $teaser, FALSE);
    }
    else {
      $item = node_prepare($item, $teaser);
    }
    
    // Allow modules to change $node->content before the node is rendered.
    node_invoke_nodeapi($item, 'view', $teaser, FALSE);
    
    // Set the proper node property, then unset unused $node property so that a
    // bad theme can not open a security hole.
    $content = drupal_render($item->content);
    if ($teaser) {
      $item->teaser = $content;
      unset($item->body);
    }
    else {
      $item->body = $content;
      unset($item->teaser);
    }
    
    // Allow modules to modify the fully-built node.
    node_invoke_nodeapi($item, 'alter', $teaser, FALSE);
  }
  
  // Allow modules to add additional item fields and/or modify $item
  $extra = node_invoke_nodeapi($item, 'rss item');
  $extra = array_merge($extra, array(array('key' => 'pubDate', 'value' => gmdate('r', $item->created)), array('key' => 'dc:creator', 'value' => $item->name), array('key' => 'guid', 'value' => $item->nid .' at '. $base_url, 'attributes' => array('isPermaLink' => 'false'))));
  foreach ($extra as $element) {
    if (isset($element['namespace'])) {
      $namespaces = array_merge($namespaces, $element['namespace']);
    }
  }
  
  // Prepare the item description
  switch ($item_length) {
  case 'fulltext':
    $item_text = $item->body;
    break;
  case 'teaser':
    $item_text = $item->teaser;
    if (!empty($item->readmore)) {
      $item_text .= '<p>'. l(t('read more'), 'node/'. $item->nid, array('absolute' => TRUE, 'attributes' => array('target' => '_blank'))) .'</p>';
    }
    break;
  case 'title':
    $item_text = '';
    break;
  }
  return format_rss_item($item->title, $item->link, $item_text, $extra);
}

/**
 * Menu callback; view a single node.
 */
function cachextra_node_page_view($node, $cid = NULL) {
  global $user;
  
  $output = cachextra('node-page-view', array('nid' => $node->nid, 'cid' => $cid), 'cachextra_node_page_view_real', array($node, $cid));
  
  if (function_exists('comment_render') && $node->comment) {
    $output .= cachextra('comments-page-view', array('nid' => $node->nid, 'cid' => $cid, 'uid' => $user->uid), 'comment_render', array($node, $cid));
  }
  
  return $output;
}

function cachextra_node_page_view_real($node, $cid = NULL){
  drupal_set_title(check_plain($node->title));
  return cachextra_node_show($node, $cid);
}

function cachextra_node_show($node, $cid, $message = FALSE) {
  if ($message) {
    drupal_set_title(t('Revision of %title from %date', array('%title' => $node->title, '%date' => format_date($node->revision_timestamp))));
  }
  $output = node_view($node, FALSE, TRUE);
  
  // Update the history table, stating that this user viewed this node.
  node_tag_new($node->nid);
  
  return $output;
}
