<?php
/** cachextra.term.inc --- 
 *
 * Copyright (C) 2009 PhoeniX11 Kayo
 *
 * Author: PhoeniX11 Kayo <phoenix11@users.sf.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

override_menu_inc('cachextra', 'taxonomy.pages.inc');
require_once('cachextra.node.inc');

function cachextra_taxonomy_term_page ($str_tids = '', $depth = 0, $op = 'page') {
  if ($op == 'page') return cachextra('taxonomy-term-page', array('tids' => $str_tids, 'page' => $_GET['page'], 'depth' => $depth),
                                      'cachextra_taxonomy_term_page_real', array($str_tids, $depth));
  if ($op == 'feed') return cachextra('taxonomy-term-feed', array('tids' => $str_tids, 'depth' => $depth),
                                      'cachextra_taxonomy_term_feed_real', array($str_tids, $depth));
  drupal_not_found();
}

function cachextra_taxonomy_term_page_real ($str_tids = '', $depth = 0) {
  cachextra_taxonomy_term_page_fake ($str_tids, $depth, 'page');
}

function cachextra_taxonomy_term_feed_real ($str_tids = '', $depth = 0) {
  cachextra_taxonomy_term_feed_fake ($str_tids, $depth, 'feed');
}

function cachextra_taxonomy_term_page_fake ($str_tids = '', $depth = 0, $op = 'page') {
  $terms = taxonomy_terms_parse_string($str_tids);
  if ($terms['operator'] != 'and' && $terms['operator'] != 'or') {
    drupal_not_found();
  }
  
  if ($terms['tids']) {
    $result = db_query(db_rewrite_sql('SELECT t.tid, t.name FROM {term_data} t WHERE t.tid IN ('. db_placeholders($terms['tids']) .')', 't', 'tid'), $terms['tids']);
    $tids = array(); // we rebuild the $tids-array so it only contains terms the user has access to.
    $names = array();
    while ($term = db_fetch_object($result)) {
      $tids[] = $term->tid;
      $names[] = $term->name;
    }
    
    if ($names) {
      $title = implode(', ', $names);
      drupal_set_title(check_plain($title));
      
      switch ($op) {
        case 'page':
          // Build breadcrumb based on first hierarchy of first term:
          $current->tid = $tids[0];
          $breadcrumb = array();
          while ($parents = taxonomy_get_parents($current->tid)) {
            $current = array_shift($parents);
            $breadcrumb[] = l($current->name, 'taxonomy/term/'. $current->tid);
          }
          $breadcrumb[] = l(t('Home'), NULL);
          $breadcrumb = array_reverse($breadcrumb);
          drupal_set_breadcrumb($breadcrumb);
          
          $output = theme('cachextra_taxonomy_term_page', $tids, taxonomy_select_nodes($tids, $terms['operator'], $depth, TRUE));
          drupal_add_feed(url('taxonomy/term/'. $str_tids .'/'. $depth .'/feed'), 'RSS - '. $title);
          return $output;
          break;
          
        case 'feed':
          $channel['link'] = url('taxonomy/term/'. $str_tids .'/'. $depth, array('absolute' => TRUE));
          $channel['title'] = variable_get('site_name', 'Drupal') .' - '. $title;
          // Only display the description if we have a single term, to avoid clutter and confusion.
          if (count($tids) == 1) {
            $term = taxonomy_get_term($tids[0]);
            // HTML will be removed from feed description, so no need to filter here.
            $channel['description'] = $term->description;
          }
          
          $result = taxonomy_select_nodes($tids, $terms['operator'], $depth, FALSE);
          $items = array();
          while ($row = db_fetch_object($result)) {
            $items[] = $row->nid;
          }
          
          cachextra_node_feed($items, $channel);
          break;
          
        default:
          drupal_not_found();
      }
    }
    else {
      drupal_not_found();
    }
  }
}

function theme_cachextra_taxonomy_term_page($tids, $result) {
  drupal_add_css(drupal_get_path('module', 'taxonomy') .'/taxonomy.css');
  
  $output = '';
  
  // Only display the description if we have a single term, to avoid clutter and confusion.
  if (count($tids) == 1) {
    $term = taxonomy_get_term($tids[0]);
    $description = $term->description;
    
    // Check that a description is set.
    if (!empty($description)) {
      $output .= '<div class="taxonomy-term-description">';
      $output .= filter_xss_admin($description);
      $output .= '</div>';
    }
  }
  
  $output .= cachextra_taxonomy_render_nodes($result);
  
  return $output;
}

function cachextra_taxonomy_render_nodes($result) {
  $output = '';
  $has_rows = FALSE;
  while ($node = db_fetch_object($result)) {
    $output .= cachextra('node-page-teaser', array('nid' => $node->nid),
                         'cachextra_node_page_teaser_real', array($node->nid));
    $has_rows = TRUE;
  }
  if ($has_rows) {
    $output .= theme('pager', NULL, variable_get('default_nodes_main', 10), 0);
  }
  else {
    $output .= '<p>'. t('There are currently no posts in this category.') .'</p>';
  }
  return $output;
}
